local VERSION = tonumber(("$Revision: 44153 $"):match("%d+"))

local _G = getfenv(0)

local friends = AceLibrary("TheSocial-1.0")

local L = AceLibrary("AceLocale-2.2"):new("AuldLangSyne_Sync")

local AuldLangSyne = AuldLangSyne
local Sync
if AuldLangSyne then
	Sync = AuldLangSyne:NewModule("Sync", "AceEvent-2.0", "AceConsole-2.0")
	if AuldLangSyne.revision < VERSION then
		AuldLangSyne.version = "r" .. VERSION
		AuldLangSyne.revision = VERSION
		AuldLangSyne.date = ("$Date: 2007-07-18 22:04:11 -0400 (Wed, 18 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
	end
else
	Sync = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0")
	AuldLangSyne_Sync = Sync -- since this isn't a module in this branch, global for access
end

function Sync:OnInitialize(name)
	local chardefaults = {
		sync = false,
	}
	local realmdefaults = {
		add_friends = {
			["*"] = {},
		},
		remove_friends = {
			["*"] = {},
		},
	}
	if AuldLangSyne then
		self.db = AuldLangSyne:AcquireDBNamespace("Sync")
		AuldLangSyne:RegisterDefaults("Sync", "char", chardefaults)
		AuldLangSyne:RegisterDefaults("Sync", "realm", realmdefaults)
	else
		self:RegisterDB("AuldLangSyneSyncDB")
		self:RegisterDefaults("realm", realmdefaults)
		self:RegisterDefaults("char", chardefaults)
	end
	
	self.menu = {
		handler = Sync,
		name = L["Sync"], type = "toggle",
		desc = L["Synchronize the friends lists of all your characters on this realm."],
		get = function() return self.db.char.sync end,
		set = function(t) self.db.char.sync = t; if t then self:SyncFriends() end end,
	}
	
	self:RegisterChatCommand({'/synctoggle',}, self.menu)
	
	if AuldLangSyne and AuldLangSyne.db.realm.add_friends then
		self.db.realm.add_friends = AuldLangSyne.db.realm.add_friends
		AuldLangSyne.db.realm.add_friends = nil
		self.db.realm.remove_friends = AuldLangSyne.db.realm.remove_friends
		AuldLangSyne.db.realm.remove_friends = nil
		self.db.realm.sync = nil
	end
end

function Sync:OnEnable(first)
	self:RegisterEvent("TheSocial10_FriendAdded")
	self:RegisterEvent("TheSocial10_FriendRemoved")
	
	if self.db.char.sync then
		self:SyncFriends()
	end
end

--function Sync:OnDisable()
--	
--end

------------------------------------------------------------------------
-- Friend syncing
------------------------------------------------------------------------

local friendCache = {}
function Sync:SyncFriends()
	local friends = self.db.realm.add_friends[self.faction]
	local remove = self.db.realm.remove_friends[self.faction]
	local playername = UnitName("player")
	
	--First, absorb any new friends from this character, and remove any friends scheduled for removal.
	for i=1, GetNumFriends() do
		local name = GetFriendInfo(i)
		if name then
			if remove[name] then
				RemoveFriend(i)
			else
				if not friends[name] then
					friends[name] = true
				end
				friendCache[name] = true
			end
		end
	end
	
	--Now add any friends this character doesn't have yet.
	local i = 1
	for name in pairs(friends) do
		if i > MAX_IGNORE then
			--Complain if we have more than 50 friends in sync. (Using MAX_IGNORE for this, since there isn't a MAX_FRIENDS.)
			self:Print(("More than %d friends are trying to be sync'd.  This won't work.  Make less friends -- what is this, some sort of social game?"):format(MAX_IGNORE))
			break
		end
		if playername ~= name and not friendCache[name] then
			AddFriend(name)
		end
		i = i + 1
	end
	
	--And clear that helpful friendCache
	for name in pairs(friendCache) do
		friendCache[name] = nil
	end
end

function Sync:TheSocial10_FriendAdded(name)
	if self.db.char.sync and name ~= UnitName("player") then
		self.db.realm.add_friends[self.faction][name] = true
		self.db.realm.remove_friends[self.faction][name] = nil -- just in case
	end
end

function Sync:TheSocial10_FriendRemoved(name)
	if self.db.char.sync and self.db.realm.add_friends[self.faction][name] ~= nil then
		self.db.realm.add_friends[self.faction][name] = nil
		self.db.realm.remove_friends[self.faction][name] = true
	end
end
